unit xocBeveL;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  ExtCtrls;

type
  TxBeveL = class(TBevel)
  private
    C_LITE : TColor;
    C_DARK : TColor;
    FBtwSpace : Integer;
    procedure _SetTopColor(val:TColor);
    procedure _SetBTMColor(val:TColor);
    procedure _SetBtwSpace(val:Integer);
  protected
    procedure Paint; override;

  public
    constructor Create(AOwner: TComponent); override;

  published
    property _Color_Lite : TColor read C_LITE write _SetTopColor;
    property _Color_Dark : TColor read C_DARK write _SetBTMColor;
    property _BtwSpace : Integer read FBtwSpace write _SetBtwSpace;
  end;

procedure Register;

implementation

//------------------------------------------------------------------------------
//------------------------------------------------------------------------------
procedure Register;
begin
  RegisterComponents('Vcl++', [TxBeveL]);
end;

//------------------------------------------------------------------------------
//------------------------------------------------------------------------------
//------------------------------------------------------------------------------
constructor TxBeveL.Create(AOwner: TComponent);
begin

Inherited Create(AOwner);

_BtwSpace:=0;
end;

//------------------------------------------------------------------------------
procedure TxBeveL.Paint;
const
  XorColor = $00FFD8CE;
var
  Color1, Color2: TColor;
  Temp: TColor;

  procedure BevelRect(const R: TRect);
  begin
    with Canvas do
    begin
      Pen.Color := Color1;
      PolyLine([Point(R.Left, R.Bottom), Point(R.Left, R.Top), Point(R.Right, R.Top)]);
      Pen.Color := Color2;
      PolyLine([Point(R.Right, R.Top), Point(R.Right, R.Bottom), Point(R.Left, R.Bottom)]);
    end;
  end;

  procedure BevelLine(C: TColor; X1, Y1, X2, Y2: Integer);
  begin
    with Canvas do
    begin
      Pen.Color := C;
      MoveTo(X1, Y1);
      LineTo(X2, Y2);
    end;
  end;

begin


  with Canvas do
  begin
    if (csDesigning in ComponentState) then
    begin
      if (Shape = bsSpacer) then
      begin
        Pen.Style := psDot;
        Pen.Mode := pmXor;
        Pen.Color := XorColor;
        Brush.Style := bsClear;
        Rectangle(0, 0, ClientWidth, ClientHeight);
        Exit;
      end
      else
      begin
        Pen.Style := psSolid;
        Pen.Mode  := pmCopy;
        Pen.Color := clBlack;
        Brush.Style := bsSolid;
      end;
    end;

    Pen.Width := 1;

    if Style = bsLowered then
    begin
      Color1 := C_DARK;//;    ++-- 
      Color2 := C_LITE;//;
    end
    else
    begin
      Color1 := C_LITE;
      Color2 := C_DARK;//; 
    end;

    case Shape of
      bsBox: BevelRect(Rect(0, 0, Width - 1, Height - 1));
      bsFrame:
        begin
          Temp := Color1;
          Color1 := Color2;
          BevelRect(Rect(1, 1, Width - 1, Height - 1));
          Color2 := Temp;
          Color1 := Temp;
          BevelRect(Rect(0, 0, Width - 2, Height - 2));
        end;
      bsTopLine:
        begin
          BevelLine(Color1, 0, 0, Width, 0);
          BevelLine(Color2, 0, 1, Width, 1);
        end;
      bsBottomLine:
        begin
          BevelLine(Color1, 0, Height - 2, Width, Height - 2);
          BevelLine(Color2, 0, Height - 1, Width, Height - 1);
        end;
      bsLeftLine:
        begin
          BevelLine(Color1, 0, 0, 0, Height);
          BevelLine(Color2, 1+FBtwSpace, 0, 1+FBtwSpace, Height);
        end;
      bsRightLine:
        begin
          BevelLine(Color1, (Width - 2)-FBtwSpace, 0, (Width - 2)-FBtwSpace, Height);
          BevelLine(Color2, Width - 1, 0, Width - 1, Height);
        end;
    end;
  end;
end;

//------------------------------------------------------------------------------
procedure TxBeveL._SetTopColor(val:TColor);
begin
C_LITE:=val;
Invalidate;
end;

//------------------------------------------------------------------------------
procedure TxBeveL._SetBTMColor(val:TColor);
begin
C_DARK:=val;
Invalidate;
end;

//------------------------------------------------------------------------------
procedure TxBeveL._SetBtwSpace(val:Integer);
begin
FBtwSpace:=val;
Invalidate;
end;

end.
