{******************************************************************************}
{**                                                                          **}
{**  TantTaskbarIcon component is an Icon for system Tray.                   **}
{**  Support ImageList for Icon images, PopUp menu.                          **}
{**  For Delphi 2-5, C++ Builder 1-4.                                        **}
{**                                                                          **}
{**  Author:    KARPOLAN                                                     **}
{**  E-Mail:    karpolan@yahoo.com , karpolan@utilmind.com                   **}
{**  Home Page: http://karpolan.i.am, http://www.utilmind.com                **}
{**  Copyright  1996-99 by KARPOLAN.                                        **}
{**  Copyright  1999, UtilMind Solutions.                                   **}
{**                                                                          **}
{******************************************************************************}
{** History:
  09 feb 1999 - 1.0 First release....(Based on my old component TaskBarIcon (1996-1998))
  10 feb 1999 - 1.1 Added ImageList support.
  13 feb 1999 - 1.3 Fixed PopUp Menu unusual bug.
  26 feb 1999 - 1.4 Compiler derectives changes. History added.
  04 mar 1999 - 1.5 Fixed new PopUp Menu bug :).
  31 aug 1999 - 1.6 Prepared for public release. Some updates...
  15 sep 1999 - 1.7 Right click bug fixed...
*******************************************************************************}
Unit xocTaskBar;

{$Include xTaskBar.Inc - contains compiler derectives}

Interface

Uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  Menus, ExtCtrls, ShellApi;

Const
  WM_TaskbarIconCallBack = WM_User + 1;

Type
  EantTaskbarIcon = Class(Exception);

{##############################################################################}
{## TantTaskbarIcon = Class(TComponent) #######################################}
{##############################################################################}
  TTaskbarIcon = Class(TComponent)
  Private
  {** Events Fields **}
    fOnMouseDown : TMouseEvent;
    fOnMouseUp   : TMouseEvent;
    fOnClick     : TNotifyEvent;
    fOnDblClick  : TNotifyEvent;
    fOnPopupMenu : TNotifyEvent;
    fOnShow      : TNotifyEvent;
    fOnHide      : TNotifyEvent;

  Protected
  {** Fields **}
    fHandle         : HWND;
    fNotifyIconData : TNotifyIconData;
    fIcon           : TIcon;
    fImageList      : TImageList;
    fImageIndex     : integer;
    fID             : integer;
    fIconFlags      : integer;
    fVisible        : Boolean;
    fEnabled        : Boolean;
    fHint           : String;
    fPopupMenu      : TPopupMenu;
    fPressed        : Boolean;
  {** Override This **}
    Procedure Notification(AComponent: TComponent; Operation : TOperation); Override;
    Procedure Loaded;                                                  Override;
  {** Messages Routine **}
    Procedure WndProc(Var Message : TMessage);                          Virtual;
  {** Taskbar Routines **}
    Procedure PrepareNotifyIconData;                                    Virtual;
    Procedure AddToTaskbar;                                             Virtual;
    Procedure DeleteFromTaskbar;                                        Virtual;
  {** ImageList Routines **}
    Procedure SetPictureByImageIndex;                                   Virtual;
  {** Events Handlers **}
    Procedure MouseDown(Button : TMouseButton; Shift: TShiftState;
                        X, Y : integer);                                Virtual;
    Procedure MouseUp(Button : TMouseButton; Shift: TShiftState;
                      X, Y : integer);                                  Virtual;
    Procedure Click;                                                    Virtual;
    Procedure DblClick;                                                 Virtual;
  {** Properties Routine **}
    Procedure SetIcon      (A : TIcon);                                 Virtual;
    Procedure SetImageList (A : TImageList);                            Virtual;
    Procedure SetImageIndex(A : integer);                               Virtual;
    Procedure SetID        (A : integer);                               Virtual;
    Procedure SetVisible   (A : Boolean);                               Virtual;
    Procedure SetEnabled   (A : Boolean);                               Virtual;
    Procedure SetHint      (A : String);                                Virtual;

  Public
  {** Init/Done **}
    Constructor Create(AOwner : TComponent);                           Override;
    Destructor  Destroy;                                               Override;
  {** Update Routine **}
    Procedure UpDate;                                                   Virtual;
  {** Misc Routine **}
    Procedure Show;                                                     Virtual;
    Procedure Hide;                                                     Virtual;
    Procedure Popup;                                                    Virtual;

  Published
  {** New Properties **}
    Property Icon            : TIcon
      Read   fIcon
      Write  SetIcon;
    Property ImageList       : TImageList
      Read   fImageList
      Write  SetImageList;
    Property ImageIndex      : integer
      Read   fImageIndex
      Write  SetImageIndex     Default 0;
    Property IconID          : integer
      Read   fID
      Write  SetID             Default 0;
    Property Visible         : Boolean
      Read   fVisible
      Write  SetVisible        Default False;
    Property Enabled         : Boolean
      Read   fEnabled
      Write  SetEnabled        Default True;
    Property Hint            : String Read   fHint Write  SetHint;
    Property PopupMenu       : TPopupMenu Read   fPopupMenu Write  fPopupMenu;
  {** New Events **}
    Property OnMouseDown     : TMouseEvent Read   fOnMouseDown Write  fOnMouseDown;
    Property OnMouseUp       : TMouseEvent
      Read   fOnMouseUp
      Write  fOnMouseUp;
    Property OnClick         : TNotifyEvent
      Read   fOnClick
      Write  fOnClick;
    Property OnDblClick      : TNotifyEvent
      Read   fOnDblClick
      Write  fOnDblClick;
    Property OnPopupMenu     : TNotifyEvent
      Read   fOnPopupMenu
      Write  fOnPopupMenu;
    Property OnShow          : TNotifyEvent
      Read   fOnShow
      Write  fOnShow;
    Property OnHide          : TNotifyEvent
      Read   fOnHide
      Write  fOnHide;
  End;{TantTaskbarIcon = Class(TComponent)}
  
Procedure Register;

{##############################################################################}
{******************************************************************************}
{##############################################################################}
Implementation
{##############################################################################}
{******************************************************************************}
{##############################################################################}
//{$R antTaskbar.res}
Procedure Register;
Begin
{** antTaskbarIcon unit **}
  RegisterComponents('Vcl++', [TTaskbarIcon]);
End;{Procedure Register}


Function GetShiftState : TShiftState;
Begin
  Result := [];
  if GetKeyState(VK_SHIFT) < 0
   then Include(Result, ssShift);
  if GetKeyState(VK_CONTROL) < 0
   then Include(Result, ssCtrl);
  if GetKeyState(VK_MENU) < 0
   then Include(Result, ssAlt);
End;{Function GetShiftState}


{##############################################################################}
{## TantTaskbarIcon = Class(TComponent) #######################################}
{##############################################################################}
Const
  strErrUpdate     = '%s.UpDate failed';
  strErrAddTo      = '%s.AddToTaskbar failed';
  strErrDeleteFrom = '%s.DeleteFromTaskbar failed';
{******************************************************************************}
{** Public ********************************************************************}
{******************************************************************************}
{** Init/Done **}
{***************}
Constructor TTaskbarIcon.Create(AOwner : TComponent);
Begin
  fIcon       := TIcon.Create;
  fImageIndex := 0;
  fID         := 0;
  fIconFlags  := NIF_TIP or NIF_MESSAGE;
  fVisible    := False;
  fEnabled    := True;
  fHint       := '';
  fPressed    := False;

  Inherited Create(AOwner);
  fHandle := AllocateHWnd(WndProc);
End;{Constructor TantTaskbarIcon.Create}

Destructor  TTaskbarIcon.Destroy;
Begin
  Visible := False;
  DeallocateHWnd(fHandle);

  Inherited Destroy;
  fIcon.Free;
End;{Destructor TantTaskbarIcon.Destroy}


{********************}
{** Update Routine **}
{********************}
Procedure TTaskbarIcon.UpDate;
Begin
  try
    PrepareNotifyIconData;
    Shell_NotifyIcon(NIM_MODIFY, @fNotifyIconData);
  except
    Raise EantTaskbarIcon.CreateFmt(strErrUpdate, [Name]);
  end;{try..except}
End;{Procedure TantTaskbarIcon.UpDate}


{******************}
{** Misc Routine **}
{******************}
Procedure TTaskbarIcon.Show;
Begin
  fIconFlags := fIconFlags or NIF_ICON;
  AddToTaskbar;
  fVisible   := True;

  if Assigned(fOnShow)
   then fOnShow(Self);
End;{Procedure TantTaskbarIcon.Show}

Procedure TTaskbarIcon.Hide;
Begin
  fIconFlags := fIconFlags and not NIF_ICON;
  DeleteFromTaskbar;
  fVisible   := False;

  if Assigned(fOnHide)
   then fOnHide(Self);
End;{Procedure TantTaskbarIcon.Hide}

Procedure TTaskbarIcon.Popup;
Var
  APoint : TPoint;
Begin
  if not Assigned(fPopupMenu)
   then Exit;

  GetCursorPos(APoint);
  fPopupMenu.Popup(APoint.X, APoint.Y);
  PostMessage(fHandle, WM_NULL, 0, 0);

  if Assigned(fOnPopupMenu)
   then fOnPopupMenu(Self);
End;{Procedure TantTaskbarIcon.Popup}


{******************************************************************************}
{** Protected *****************************************************************}
{******************************************************************************}
{** Override This **}
{*******************}
Procedure TTaskbarIcon.Notification(AComponent: TComponent;
                                       Operation : TOperation);
Begin
  if (AComponent = fPopupMenu) and (Operation = opRemove)
   then fPopupMenu := nil;

  Inherited Notification(AComponent, Operation);
End;{Procedure TantTaskbarIcon.Notification}

Procedure TTaskbarIcon.Loaded;
Begin
  Inherited Loaded;
  if fVisible
   then Show;
End;{Procedure TantTaskbarIcon.Loaded}

{**********************}
{** Messages Routine **}
{**********************}
Procedure TTaskbarIcon.WndProc(Var Message: TMessage);
Var
  APoint : TPoint;
Begin
  with Message do
   begin
     if (Msg = WM_TaskbarIconCallBack) and fEnabled then
      begin
        GetCursorPos(APoint);

        case LParam of
         WM_LBUTTONDOWN   : begin {** Left Button Down **}
           fPressed := True;
           MouseDown(mbLeft, GetShiftState + [ssLeft], APoint.X, APoint.Y);
         end;{case of WM_LBUTTONDOWN}
         WM_LBUTTONUP     : begin {** Left Button Up **}
           MouseUp(mbLeft, GetShiftState + [ssLeft], APoint.X, APoint.Y);
           if fPressed
            then Click;
           fPressed := False;
         end;{case of WM_LBUTTONDOWN}
         WM_LBUTTONDBLCLK : DblClick;

         WM_RBUTTONDOWN   : begin
           SetForegroundWindow(fHandle); // needs for hiding Popup on second click
           MouseDown(mbRight, GetShiftState + [ssRight], APoint.X, APoint.Y);
         end;{case of WM_RBUTTONDOWN}
         WM_RBUTTONUP     : begin {** Right Button Up = PopUp Menu **}
           Popup;
           MouseUp(mbRight, GetShiftState + [ssRight], APoint.X, APoint.Y);
         end;{case of WM_RBUTTONUP}
        end;{case LParam of}

      end{if Msg = WM_TaskbarIconCallBack then}
     else Result := DefWindowProc(fHandle, Msg, wParam, lParam);
   end;{with Message do}
End;{Procedure TantTaskbarIcon.WndProc}

{**********************}
{** Taskbar Routines **}
{**********************}
Procedure TTaskbarIcon.PrepareNotifyIconData;
Begin
  with fNotifyIconData do
   begin
     cbSize           := SizeOf(TNotifyIconData);
     Wnd              := fHandle;
     uID              := fID;
     uFlags           := fIconFlags;
     uCallbackMessage := WM_TaskbarIconCallBack;
     hIcon            := fIcon.Handle;
     StrCopy(szTip, PChar(fHint));
   end;{with fNotifyIconData do}
End;{Procedure TantTaskbarIcon.PrepareNotifyIconData}

Procedure TTaskbarIcon.AddToTaskbar;
Begin
  try
    PrepareNotifyIconData;
    Shell_NotifyIcon(NIM_ADD, @fNotifyIconData);
  except
    Raise EantTaskbarIcon.CreateFmt(strErrAddTo, [Name]);
  end;{try..except}
End;{Procedure TantTaskbarIcon.AddToTaskbar}

Procedure TTaskbarIcon.DeleteFromTaskbar;
Begin
  try
    PrepareNotifyIconData;
    Shell_NotifyIcon(NIM_DELETE, @fNotifyIconData);
  except
    Raise EantTaskbarIcon.CreateFmt(strErrDeleteFrom, [Name]);
  end;{try..except}
End;{TantTaskbarIcon.DeleteFromTaskbar}


{************************}
{** ImageList Routines **}
{************************}
Procedure TTaskbarIcon.SetPictureByImageIndex;
Begin
  if Assigned(fImageList) then
   begin
     fIcon.Assign(nil);
     fImageList.GetIcon(fImageIndex, fIcon);
     Update;
   end;{if Assigned(fImageList) then}
End;{Procedure TantTaskbarIcon.SetPictureByImageIndex}


{*********************}
{** Events Handlers **}
{*********************}
Procedure TTaskbarIcon.MouseDown(Button : TMouseButton; Shift: TShiftState;
                                    X, Y : integer);
Begin
  if Assigned(fOnMouseDown)
   then fOnMouseDown(Self, Button, Shift, X, Y);
End;{Procedure TantTaskbarIcon.MouseDown}

Procedure TTaskbarIcon.MouseUp(Button : TMouseButton; Shift: TShiftState;
                                  X, Y : integer);
Begin
  SetForegroundWindow(Application.Handle);
  if Assigned(fOnMouseUp)
   then fOnMouseUp(Self, Button, Shift, X, Y);
End;{Procedure TantTaskbarIcon.MouseUp}

Procedure TTaskbarIcon.Click;
Begin
  if Assigned(fOnClick)
   then fOnClick(Self);
End;{Procedure TantTaskbarIcon.Click}

Procedure TTaskbarIcon.DblClick;
Begin
  if Assigned(fOnDblClick)
   then fOnDblClick(Self);
End;{Procedure TantTaskbarIcon.DblClick}


{************************}
{** Properties Routine **}
{************************}
Procedure TTaskbarIcon.SetIcon(A : TIcon);
Begin
  fIcon.Assign(A);
  Update;
End;{Procedure TantTaskbarIcon.SetIcon}

Procedure TTaskbarIcon.SetImageList(A : TImageList);
Begin
  fImageList := A;
  SetPictureByImageIndex;
End;{Procedure TantTaskbarIcon.SetImageList}

Procedure TTaskbarIcon.SetImageIndex(A : integer);
Begin
  fImageIndex := A;
  SetPictureByImageIndex;
End;{Procedure TantTaskbarIcon.SetImageIndex}

Procedure TTaskbarIcon.SetID(A : integer);
Var
  RegBool : Boolean;
Begin
  if A = fID
   then Exit;

  RegBool := Visible;
  Visible := False;
  fID     := A;
  Visible := RegBool;
End;{Procedure TantTaskbarIcon.SetID}

Procedure TTaskbarIcon.SetVisible(A : Boolean);
Begin
  if fVisible = A
   then Exit;

  fVisible := A;
  if fVisible
   then Show
   else Hide;
End;{Procedure TantTaskbarIcon.SetVisible}

Procedure TTaskbarIcon.SetEnabled(A : Boolean);
Begin
  fEnabled := A;
End;{Procedure TantTaskbarIcon.SetEnabled}

Procedure TTaskbarIcon.SetHint(A : String);
Begin
  fHint := A;
  if Length(fHint) > 63
   then SetLength(fHint, 63);
  Update;
End;{Procedure TantTaskbarIcon.SetHint}


END{Unit antTaskbarIcon}.
